#define FUNC symlink
#define SFUNC "symlink"
#define FUNC_STR "\"%s\", \"%s\""
#define FUNC_IMP oldpath, newpath
#define ARG_CNT 2
#define ARG_USE "<path>(old) <path>(new)"

#define process_args() \
	s = argv[i++]; \
	char *oldpath = s; \
	\
	s = argv[i++]; \
	char *newpath = s;

#include "test-skel-0.c"
